#
# Easysim::Communication.pm
#
# 2001-12-02: Ivan Malich, malich@decef.elf.stuba.sk
#

package Easysim::Communication;

use strict;
use warnings;
use IO::Socket;
use Net::hostent;
use Easysim::Utils;
use Easysim::XML;


################################################################################
#                                                                              #
#                           "PUBLIC" FUNCTIONS                                 #
#                                                                              #
################################################################################

sub accept_connection
{
	my $self=shift;
	die "not a server end point!\n" if($self->{'type'} ne "server");
	my ($rin,$rout);
	vec($rin='',fileno($self->{'socket'}),1)=1;
	my ($connection)=select($rout=$rin,undef,undef,0.1);
	return(undef) unless($connection);# no new connection

	# we have new connection - create an object for it
	my $channel={};
	bless($channel);
	my $socket=$self->{'socket'}->accept();
	$channel->{'socket'}=$socket;
	$channel->{'type'}="client";
	$channel->{'host'}=$socket->peerhost;
	$channel->{'port'}=$socket->peerport;
	return($channel);
}

################################################################################
# $connection=new($type,$address);
#
# Creates new tcp client or server.
# $type - type of servire, $type = "server" | "client"
# $address - address of service, $address = "port" | "hostname:port"
#
sub new
{
	my $package=shift;
	my ($type,$host)=@_;
	my $self={};
	bless($self);
	if($type eq "server"){
		$self->create_server($host);
	}elsif($type eq "client"){
		$self->create_client($host);
	}else{
		my ($cfunc,$cfile,$cline)=caller;
		warn "Bad type of service: '$type' when calling package $package from file $cfile, line $cline.\n";
		return(0);
	}
	return($self);
}

sub receive_message
{
	my ($channel)=@_;
	my ($socket,$buffer)=($channel->{'socket'},\$channel->{'buffer'});
	my ($rin,$rout);
	vec($rin='',fileno($socket),1)=1;
	my ($message)=select($rout=$rin,undef,undef,0.1);
	if($message){# something new in socket
		my $buf;
		my $nread=sysread($socket,$buf,4096);
		if($nread){# something from our party
			$$buffer.=$buf;
		}else{# closed socket
			return(0);
		}
	}
	no warnings;
	if($$buffer=~/<\?xml\s+.*?\?>\s*<easysim>.*?<\/easysim>/s){
		# some complete message available!
		use warnings;
		my $m=substr($$buffer,$-[0],$+[0]-$-[0]);
		my $r=substr($$buffer,$+[0],length($$buffer)-$+[0]);
		$$buffer=$r;

		my ($parser,$dom);
		# try to transform a message into XML object
		eval{
			$parser=Easysim::XML->new();
			$dom=$parser->parse_string("$m");
		};
		return($@ ? undef : $dom);# errors?
	}
	return(undef);
}

sub send_message
{
	my ($channel,$dom)=@_;
	my $socket=$channel->{'socket'};
	print $socket $dom->toString;
	return(1);
}

sub shutdown
{
}

################################################################################
#                                                                              #
#                           "PRIVATE" FUNCTIONS                                #
#                                                                              #
################################################################################

################################################################################
# $success=create_client($address);
#       
# Connects to tcp server.
# $address - specification of server, $address = "hostname:port";
# 
sub create_client
{
	my $self=shift;
	my ($addr)=@_;
	no warnings;
	my ($host,$tcp_port)=($addr=~/^(.*?):(.*)$/);
	die "Unable to connect to '$host:$tcp_port'\n" if(!$host && !$tcp_port);
	use warnings;

	my $socket=IO::Socket::INET->new(
		"Proto"         =>"tcp",
		"PeerAddr"      =>$host,
		"PeerPort"      =>$tcp_port,
	);
	die "Unable to connect to $host:$tcp_port\n" unless($socket);
	$socket->autoflush(1);
	$self->{'host'}=$host;
	$self->{'port'}=$tcp_port;
	$self->{'socket'}=$socket;
	$self->{'type'}="client";
}

################################################################################
# $success=create_server($tcp_port);
#
# Creates an tcp server listening on $tcp_port.
#
sub create_server
{
	my $self=shift;
	my ($tcp_port)=@_;
	$tcp_port=~s/^.*?:?//;
	my $socket=IO::Socket::INET->new(
		"Proto"         =>'tcp',
		"LocalPort"     =>$tcp_port,
		"Listen"        =>SOMAXCONN,
		"Reuse"         =>1,
	);
	warn "Unable to allocate port $tcp_port\n" unless($socket);
	$socket->autoflush(1);
	warn "port $tcp_port allocated...\n";
	$self->{'port'}=$tcp_port;
	$self->{'socket'}=$socket;
	$self->{'type'}="server";
}

1;

#
# END Easysim::Communication.pm
#

