#
# Easysim::Compiler.pm
#
# 2001-12-10: Ivan Malich, malich@decef.elf.stuba.sk
#

package Easysim::Compiler;

sub NAME{"Easysim::Compiler";}
sub VERSION{"2001-12-15";}

use strict;
use warnings;
use Easysim::Communication;
use Easysim::Utils;
use Easysim::XML;

################################################################################
# ($name,$version)=module_check();
#
# Returns name and version of module.
#
sub module_check
{
	return(NAME,VERSION);
}

################################################################################
# module_entry($channel,$dom);
#
# This is the entry point to module Easysim::Compiler. All actions in this
# module should start from here.
# $channel - communication object
# $dom - a DOM object containing an VHDL code from GUI
#
sub module_entry
{
	my ($channel,$data)=@_;

=c
	Now, proceed input $data, validate VHDL source, parse it, create
	internal structure for simulator, create signal list for GUI.
=cut
	# this is output for both GUI and simulator
	print "Compiler: sending answer...\n";
	my $msg=qq~<?xml version="1.0" encoding="iso8859-2"?>
		<easysim>
		    <compiler>
			<msg type="status" status="success" command="compile"/>
			<msg type="signals" status="note">
			    <entity name="counter">
				<architecture name="behaviour">
				    <process name="count_up">
					<signal name="SIG_1" type="bit"/>
					<signal name="SIG_2" type="bit"/>
					<signal name="SIG_3" type="bit"/>					
				    </process>
				</architecture>
			    </entity>
			</msg>
			<msg type="vhdl" status="note">
				...nieco pre simulator...
			</msg>
		    </compiler>
		</easysim>
	~;
	my $parser=Easysim::XML->new();
	my $dom_reply=$parser->parse_string("$msg");
	$channel->send_message($dom_reply);

	while(1){
		my $dom=$channel->receive_message();
		if(defined $dom and not $dom){
			# closed kernel's socket - shut down compiler
			print "Compiler: Kernel $channel disconnected, exitting...\n";
			exit;
		}elsif(defined $dom){
			# some message!
			print "Compiler: sending answer...\n";
			$channel->send_message($dom_reply);
		}
	}
}

1;

#
# END Easysim::Compiler.pm
#
