#
# Easysim::Modules.pm
#
# 2001-12-02: Ivan Malich, malich@decef.elf.stuba.sk
#

package Easysim::Modules;

use strict;

################################################################################
# ($name,$version,$error)=run_compiler($config,$proj,$dom);
#
# Tries to run compiler module.
# $name,$version - on success sets to appropriate values, sets to undef otherwise
# $error - on error, sets this to error message
#
sub run_compiler
{
	my ($config,$proj,$dom)=@_;
	# check, whether it is possible to load module
	my ($m_name,$m_version);
	{
		eval{
			my $module="$config->{'module-compiler'}.pm";
			my $checker="$config->{'module-compiler'}::$config->{'module-checker'}";
			$module=~s/::/\//g;
			require "$module";
			no strict;
			($m_name,$m_version)=&$checker();
		};
	};
	print $@;
	return(undef,undef,$@) if($@);
	print "Kernel: Loading module $m_name, version $m_version\n";
	# we don't need zombies - ignore the child death
	$SIG{'CHLD'}="IGNORE";
	if(fork()){
		# parent - return back
		return($m_name,$m_version);
	}
	# child - proceed the compiler
	# first, register module in kernel
	my $kernel=new Easysim::Communication("client","localhost:$config->{'port-compiler'}");
	sleep(1);
	my $msg=qq~<?xml version="1.0" encoding="iso8859-2"?>
		<easysim><project id="$proj->{'id'}"><commands>
		<command name="registerCompiler" module="$m_name" version="$m_version"/>
		</commands></project></easysim>
	~;
	my $parser=Easysim::XML->new();
	my $reg_dom=$parser->parse_string("$msg");
	$kernel->send_message($reg_dom);

	# wait for confirmation from kernel
	$reg_dom=undef;
	while(not defined $reg_dom){
		$reg_dom=$kernel->receive_message();
	}
	die "Error: Compiler: Unable to register module.\n" if(not $reg_dom);

	my $root=$reg_dom->getDocumentElement();
	my @project=$root->findnodes('/easysim/project');
	my @msg=$root->findnodes('/easysim/project/messages/kernel/msg');
	if(
		scalar(@project) != 1
		or $project[0]->getAttribute("id") ne $proj->{'id'}
		or scalar(@msg) != 1
		or $msg[0]->getAttribute("type") ne "status"
		or $msg[0]->getAttribute("status") ne "success"
		or $msg[0]->getAttribute("command") ne "registerCompiler"
	){
		die "Error: Compiler: Registration refused by kernel.\n";
	}

	# compiler successfully connected, run the module itself
	# just a little magis with command line...
	$0.=" --module=$config->{'module-compiler'}";
	my $module="$config->{'module-compiler'}.pm";
	my $module_entry="$config->{'module-compiler'}::$config->{'module-entry'}";
	$module=~s/::/\//g;
	require "$module";
	no strict;
	&$module_entry($kernel,$dom);
	exit;
}

################################################################################
# ($name,$version,$error)=run_simulator($config,$proj,$dom);
#
# Tries to run simulator module.
# $name,$version - on success sets to appropriate values, sets to undef otherwise
# $error - on error, sets this to error message
#
sub run_simulator
{
	my ($config,$proj,$dom)=@_;
	# check, whether it is possible to load module
	my ($m_name,$m_version);
	{
		eval{
			my $module="$config->{'module-simulator'}.pm";
			my $checker="$config->{'module-simulator'}::$config->{'module-checker'}";
			$module=~s/::/\//g;
			require "$module";
			no strict;
			($m_name,$m_version)=&$checker();
		};
	};
	print $@;
	return(undef,undef,$@) if($@);
	print "Kernel: Loading module $m_name, version $m_version\n";
	# we don't need zombies - ignore the child death
	$SIG{'CHLD'}="IGNORE";
	if(fork()){
		# parent - return back
		return($m_name,$m_version);
	}
	# child - proceed the compiler
	# first, register module in kernel
	my $kernel=new Easysim::Communication("client","localhost:$config->{'port-simulator'}");
	sleep(1);
	my $msg=qq~<?xml version="1.0" encoding="iso8859-2"?>
		<easysim><project id="$proj->{'id'}"><commands>
		<command name="registerSimulator" module="$m_name" version="$m_version"/>
		</commands></project></easysim>
	~;
	my $parser=Easysim::XML->new();
	my $reg_dom=$parser->parse_string("$msg");
	$kernel->send_message($reg_dom);

	# wait for confirmation from kernel
	$reg_dom=undef;
	while(not defined $reg_dom){
		$reg_dom=$kernel->receive_message();
	}
	die "Error: Simulator: Unable to register module.\n" if(not $reg_dom);

	my $root=$reg_dom->getDocumentElement();
	my @project=$root->findnodes('/easysim/project');
	my @msg=$root->findnodes('/easysim/project/messages/kernel/msg');
	if(
		scalar(@project) != 1
		or $project[0]->getAttribute("id") ne $proj->{'id'}
		or scalar(@msg) != 1
		or $msg[0]->getAttribute("type") ne "status"
		or $msg[0]->getAttribute("status") ne "success"
		or $msg[0]->getAttribute("command") ne "registerSimulator"
	){
		die "Error: Simulator: Registration refused by kernel.\n";
	}

	# compiler successfully connected, run the module itself
	# just a little magis with command line...
	$0.=" --module=$config->{'module-simulator'}";
	my $module="$config->{'module-simulator'}.pm";
	my $module_entry="$config->{'module-simulator'}::$config->{'module-entry'}";
	$module=~s/::/\//g;
	require "$module";
	no strict;
	&$module_entry($kernel,$dom);
	exit;
}

1;

#
# END Easysim::Communication.pm
#
