#
# Easysim::Simulator.pm
#
# 2001-12-10: Ivan Malich, malich@decef.elf.stuba.sk
#

package Easysim::Simulator;

sub NAME{"Easysim::Simulator";}
sub VERSION{"2001-12-15";}

use strict;
use warnings;
use Easysim::Communication;
use Easysim::Utils;
use Easysim::XML;

################################################################################
# ($name,$version)=module_check();
#
# Returns name and version of module.
#
sub module_check
{
	return(NAME,VERSION);
}

################################################################################
# module_entry($channel,$dom);
#
# This is the entry point to module Easysim::Compiler. All actions in this
# module should start from here.
# $channel - communication object
# $dom - a DOM object containing an VHDL code from GUI
#
sub module_entry
{
	my ($channel,$data)=@_;

	# this is output for GUI 
	print "Simulator: sending answer...\n";
	my $msg=qq~<?xml version="1.0" encoding="iso8859-2"?>
		<easysim>
		    <simulator>
			<msg type="status" status="success" command="simulate" timeFrom="0" timeTo="10ns"/>
			<msg type="waveform">
			    <entity name="counter">
				<architecture name="behaviour">
				    <process name="count_up">
					<signal name="SIG_1" type="bit">
					    <change time="3ns" newValue="1"/>
					    <change time="4ns" newValue="0"/>
					    <change time="7ns" newValue="1"/>
					    <change time="10ns" newValue="0"/>
					</signal>
					<signal name="SIG_2" type="bit">
						<change time="2ns" newValue="1"/>
					    <change time="5ns" newValue="0"/>
					    <change time="6ns" newValue="1"/>
					</signal>
					<signal name="SIG_3" type="bit" />
				    </process>
				</architecture>
			    </entity>
			</msg>
		    </simulator>
		</easysim>
	~;
	my $parser=Easysim::XML->new();
	my $dom_reply=$parser->parse_string("$msg");
	$channel->send_message($dom_reply);

	while(1){
		my $dom=$channel->receive_message();
		if(defined $dom and not $dom){
			# closed kernel's socket - shut down simulator
			print "Simulator: Kernel $channel disconnected, exitting...\n";
			exit;
		}elsif(defined $dom){
			# some message!
			print "Simulator: sending answer...\n";
			$channel->send_message($dom_reply);
		}
	}
}

1;

#
# END Easysim::Simulator.pm
#

