#
# Easysim::Simulator.pm
#	- main body of program that hadles communication with kernel
#
# 14.12.2001: Martin Bôbik, bobik@decef.elf.stuba.sk
#
# TEAM-08-

package Easysim::Simulator;

use strict;
use warnings;
use Easysim::XML;
use EasySim::Communication;
use Easysim::Utils;
use Easysim::SimuLib;
use Data::Dumper;

################################################################################
# module_entry($channel,$dom);
#
# This is the entry point to module Easysim::Simulator. All actions in this
# module should start from here.
# $channel - communication object
# $dom - a DOM object containing parsed VHDL code
#
sub module_entry
{
	my ($channel,$dom)=@_;
	my ($model, $sim, $scheduler, $mesg, $parser, $projectID);
	
	# hello that's mee ... simulator	
	#$channel->send_message($dom);
	
	$model = new TModel;
	
	# first receive Internal Model from Compiler modul
	#
	# $dom=$channel->receive_message();
	# $mesg = $dom->toString;
	# $Model->Load($mesg); not included in prototype
	
	$scheduler = new TScheduler;
	$sim = new TTimeSimulator($model, $scheduler);

	
	if(defined $dom and not $dom){
		# hmmm, closed socket
		#die "Error: closed socket to kernel.\n";
	}elsif(defined $dom){
		print "new message from kernel in \$dom.\n";
	}

	my ($simTime, @result) = (100, undef);
	
	# Next receive request from GUI to simulate model
	#
	$dom=undef;
	$dom=$channel->receive_message() while (! defined $dom);
	$mesg= $dom->toString;
	$channel->send_message($dom);

=tomuto modulu pride napr. nieco taketo:

<easysim>
	<project id="10">
 <commands>
<command name="simulate" timeFrom="0ns" timeTo="15ns">
				<!-- zoznam signalov, ktorych hodnoty budeme posielat do GUI -->
				<entity name="">
					<architecture name="">
						<process name="">
							<signal name="sig1"/>
							<signal name="sig2"/>
					</architecture>
				</entity>
			</command>
</commands>
</project>
</easysim>
);
=cut


	($simTime)= ($mesg =~ /timeTo="(\d+)ns"/);
	($projectID)=($mesg =~ /project id="(\d+)"/);
	
	my @signals=($mesg=~/signal name="(.*)"/gs);
	$sim->Start($simTime);
	@result = $sim->GetResults(@signals);

	#print Dumper(@result);

	
=ja budem posielat napr. nieco taketo:

<easysim>
	<project id="10">
		<simulator>
			<signal name="sig1">
				<time time="3ns" newValue="1"/>
				<time time="4ns" newValue="0"/>
				<time time="6ns" newValue="0"/>
			</signal>
			<signal name="sig2">
				<time time="4ns" newValue="0"/>
			</signal>
		</simulator>
	</project>
</easysim>

=cut


	$dom = Easysim::XML::Document->new("1.0","iso8859-2");
	my $easysim=$dom->createElement("easysim");
	$dom->setDocumentElement($easysim);
	my $project=$dom->createElement("project");
	$easysim->appendChild($project);
	$project->setAttribute("id","$projectID");
	my $simulator=$dom->createElement("simulator");
	$project->appendChild($simulator);
	
	my ($signal, $tmp, $time) = (undef, "",undef);

	for( my $i=0; $i < @result; $i++)
	{
		if ($tmp ne $result[$i]{name})
		{
			$signal=$dom->createElement("signal");
			$signal->setAttribute("name", "$result[$i]{name}");
			$simulator->appendChild($signal);
			$time = $dom->createElement("time");
			$time->setAttribute("time", "$result[$i]{timepos}");
			$time->setAttribute("newValue", "$result[$i]{value}");
			$signal->appendChild($time);
			$tmp = $result[$i]{name};
		} else {
			$time = $dom->createElement("time");
			$time->setAttribute("time", "$result[$i]{timepos}");
			$time->setAttribute("newValue", "$result[$i]{value}");
			$signal->appendChild($time);
		}			
	
	}
	
	Easysim::Utils::microsleep(0.25);# sleep 1/4 sec.
	
	# send simulate results to GUI
	#
	$channel->send_message($dom);

	return();# exitting this funcrion means shutting down whole module
}

1;

####################################################
# class TTimeSimulator
#	- Main object responsible of process time model
#
#

package TTimeSimulator;

   sub new {
        my $proto = shift;
        my $class = ref($proto) || $proto;
	my $mod = shift;
	my $sch = shift;
        my $self  = {
            Model       => \$mod,
            Scheduler   => \$sch,
	    EndTime		=> 0,
	    Result	=> undef,
        };
        bless ($self, $class);
        return $self;
    }

    sub Start {
        my $self = shift;
	
        if (@_) {
            $self->{EndTime} = shift;
        }
        return $self->{Class};
    }

    sub RunProcess {
        my $self = shift;
        if (@_) {
            print "\n$self->{Class} > Not supported in prototype !";
        }
        return $self->{Class};
    }

    sub GetResults(@) {
        my $self = shift;
        if (@_) {
            #print "\n$self->{Class} > Not supported in prototype !";
        }
	my (@res, @sigNames);
	while (@_)
	{
		push (@sigNames, shift);
	};
	
	for (my $i=0; $i < @sigNames; $i++)
	{
		my $changes = rand(20);
		for (my $j=0; $j < $changes; $j++)
		{
			push @res, {
				name => $sigNames[$i],
				timepos => int(rand($self->{EndTime})),
				value => (0,1)[rand(2)]
			};
		}	
	}
	
        return @res;
    }
    
    sub UserValues {
        my $self = shift;
        if (@_) {
            print "\n$self->{Class} > Not supported in prototype !";
        }
        return $self->{Class};
    }

####################################################

#
# END Easysim::Simulator.pm
#

