package Easysim::XML::Document;

use strict;
use Easysim::XML::Element;

################################################################################
# $element=$dom->createElement($name);
#
# Creates new DOM element.
# $element - new created element
# $dom - the DOM object
# $name - name of the new DOM element
#
sub createElement
{
	my ($dom,$name)=@_;
	my $element=Easysim::XML::Element->new($name);
	return($element);
}

################################################################################
# $element=$dom->getDocumentElement();
#
# Returns the root node in this DOM document.
#
sub getDocumentElement
{
	my ($dom)=@_;
	return($dom->{'root'});
}

################################################################################
# $dom=new($version,$encoding);
#
# Creates new XML document;
# $version - currently "1.0"
# $encoding - currently "iso8859-2"
#
sub new
{
	my ($package,$version,$encoding)=@_;
	my $dom={};
	bless($dom);
	$dom->{'version'}=$version;
	$dom->{'encoding'}=$encoding;
	return($dom);
}

################################################################################
# $dom->setDocumentElemen($element);
#
# Sets the root element of the DOM object.
# $dom - the DOM object
# $element - DOM element set to be a root element
#
sub setDocumentElement
{
	my ($dom,$element)=@_;
	$dom->{'root'}=$element;
}

################################################################################
# $dom->toString($level,$step);
#
# Converts the DOM object to string;
# $dom - the converted DOM object
# $level,$step - when dumping the XML string, uses $level x $step for indenting
# of each XML tag. Set both to undef to disable this feature.
#
sub toString
{
	my ($dom,$level,$step)=@_;
	my $element=$dom->{'root'};
	die "Error: toString: Root element not set.\n" if(not defined $element or not $element);
	my $ret=$element->toString($level,$step);
	my $header=qq~<?xml version="$dom->{'version'}" encoding="$dom->{'encoding'}"?>~;
	return("$header\n$ret");
}

1;

