package Easysim::XML::Element;

use strict;

################################################################################
# $element->appendChild($child);
#
# Appends new element to parent.
# $element - the parent
# $child - the child
#
sub appendChild
{
	my ($element,$child)=@_;
	# chechk, whether not connected already
	if(ref($element->{'child'}) eq "ARRAY" and scalar(@{$element->{'child'}})){
		foreach (@{$element->{'child'}}){
			return if($child eq $_);
		}
	}
	push @{$element->{'child'}},$child;
	$child->{'parent'}=$element;
}

################################################################################
# $element->appendTextNode($test);
#
# Adds the text to the element.
#
sub appendTextNode
{
	my ($element,$text)=@_;
	$element->{'text'}=$text;
}

sub findnodes
{
	my ($element,$path)=@_;
	die "Error: findnodes: Wrong path: '$path'\n" if($path=~/\/\/\//);
	$path=~s/[\/]*$//;
	if($path=~/^[^\/]/){
		# look for direct children only
		my ($child_name,$next_path)=($path=~/^([^\/]*)\/?(.*)/);
		$next_path=~s/^\//\/\//;
		my @nodes;
		for my $n ($element->getChildNodes()){
			if($child_name eq $n->getName){
				if(not $next_path){
					# OK, no next nodes to find
					push @nodes,$n;
				}elsif($next_path=~/^\/\//){
					# next search for any child (//)
					print "next: $next_path\n";
					print "NOT IMPLEMENTED YET\n";
				}else{
					# next search only direct children
					push @nodes,$n->findnodes($next_path);
				}
			}
		}
		return(@nodes);
	}elsif($path=~/^\/[^\/]/){
		# look for root and then for children only
		my ($root_name,$next_path)=($path=~/^\/([^\/]*)\/?(.*)/);
		$next_path=~s/^\//\/\//;
		# find the root node
		my $tmp=$element->getParentNode;
		while($tmp){
			$element=$tmp;
			my $tmp=$element->getParentNode;
		}
		my @nodes;
		if($root_name eq $element->getName){
			if(not $next_path){
				# OK, no next nodes to find
				push @nodes,$element;
			}elsif($next_path=~/^\/\//){
				# next search for any child (//)
				print "next: $next_path\n";
				print "NOT IMPLEMENTED YET\n";
			}else{
				# next search only direct children
				push @nodes,$element->findnodes($next_path);
			}
		}
		return(@nodes);
	}elsif($path=~/^\/\/[^\/]/){
		# look for root and then for any of children
		my ($child_name,$next_path)=($path=~/^([^\/]*)\/?(.*)/);
		$next_path=~s/^\//\/\//;
		print "next: $next_path\n";
		print "NOT IMPLEMENTED YET\n";
		return();
	}
}

################################################################################
# @names=$element->getAttributes();
#
# Returns all attributes of element.
# @names - array of attributes names
# $element - the parent element
#
sub getAttributes
{
	my ($element)=@_;
	my @names;
	if(ref($element->{'attr'}) eq "HASH" and scalar(keys %{$element->{'attr'}})){
		foreach (keys %{$element->{'attr'}}){
			push @names,$_;
		}
	}
	return(@names);
}

################################################################################
# $value=$element->getAttribute($name);
#
# Returns a value of given attribute of element.
# $value - the value of attribute
# $element -the given element
# $name - name of attribute
#
sub getAttribute
{
	my ($element,$name)=@_;
	return($element->{'attr'}{$name});
}

################################################################################
# @children=$element->getChildNodes();
#
# Returns all child nodes of $element.
#
sub getChildNodes
{
	my ($element)=@_;
	if(ref($element->{'child'}) eq "ARRAY" and scalar(@{$element->{'child'}})){
		return(@{$element->{'child'}});
	}
	return();
}

################################################################################
# $data=$element->getData();
#
# Returns text data stored in DOM element.
#
sub getData
{
	my ($element)=@_;
	return($element->{'text'});
}

################################################################################
# $name=$element->getName();
#
# Returns the element name.
#
sub getName
{
	my ($element)=@_;
	return($element->{'name'});
}

################################################################################
# $parent=$element->getParentNode();
#
# Returns a parent of $element of undef.
#
sub getParentNode
{
	my ($element)=@_;
	return($element->{'parent'});
}

################################################################################
# $element=new($name);
#
# Creates new DOM element;
# $element - created element
# $name - name of the new element
#
sub new
{
	my ($package,$name)=@_;
	my $element={};
	bless($element);
	$element->{'name'}=$name;
	return($element);
}

################################################################################
# $element->setAttribute($name,$value);
#
# Sets the value of attribute fo element.
# $element - the given DOM element
# $name - the name of set attribute
# $value - the value of attribute
#
sub setAttribute
{
	my ($element,$name,$value)=@_;
	$element->{'attr'}{$name}=$value;
}

################################################################################
# $element->toString($level,$step);
#
# Converts the DOM element to string;
# $element - the converted DOM element
# $level,$step - when dumping the XML string, uses $level x $step for indenting
# of each XML tag. Set both to undef to disable this feature.
#
sub toString
{
	my ($element,$level,$step)=@_;
	my ($name,$ret);
	$step = defined $step ? $step : "  ";
	my ($s1,$s2,$next)=defined $level ? ($step x $level,"\n",$level+1) : ();
	if(defined $element and defined $element->{'name'}){
		$name=$element->{'name'};
	}
	my @attrs;
	for my $k ($element->getAttributes()){
		my $v=$element->getAttribute($k);
		push @attrs,"$k=\"$v\"";
	}
	my $attr=join(" ",@attrs);
	$attr=" $attr" if($attr);
	my $text=$element->getData();
	if(ref($element->{'child'}) eq "ARRAY" and scalar(@{$element->{'child'}})){
		$ret="$ret$s1<$name$attr>$text$s2";
		for my $e (@{$element->{'child'}}){
			$ret=$ret.$e->toString($next,$step);
		}
		$ret="$ret$s1</$name>$s2";
	}elsif(not $text=~/^$/){
		$ret="$ret$s1<$name$attr>$text</$name>$s2";
	}else{
		$ret="$ret$s1<$name$attr/>$s2";
	}
	return($ret);
}

1;

