#!/usr/bin/perl
#
# EasySim GUI
#
# 2001-12-16
# Norbert Ordog <nord@nord.sk>, Team 08
#

use Tk;
use Tk::BrowseEntry;
use Tk::HList;
use Tk::ItemStyle;
use Tk::LabFrame;

use Easysim::Communication;
use Easysim::Message;
use Easysim::Utils;
use Easysim::XML;

#$DEBUG = 1;	# Debug mode with predefined signals

# ---- Constant variables -------
$SIGNAL_HEIGHT = 25; 	# height of signal
$SIGNAL_PADDING = 5;	# padding from canvas borders
$SIGNAL_REAL_HEIGHT = $SIGNAL_HEIGHT - 2 * $SIGNAL_PADDING;
$SIGNAL_COLOR = DarkBlue;
# -------------------------------

# ---- Global Style Sheet ----
my @FRAME_PACK_STYLE = (-padx => 10, -pady => 5);
my @ENTRY_PACK_STYLE = (-padx => 10, -pady => 5);
my @BUTTON_STYLE = (width => 10);
my @BUTTON_PACK_STYLE = (-padx => 10, -pady => 5);
# ----------------------------

# ---- Communication config ----
my $KERNEL_SERVER = "localhost";
my %config=(
	"port-gui"		=>2467,
	"port-compiler"		=>2468,
	"port-simulator"	=>2469,
);
my $config=\%config;
# ------------------------------

$sim_time = 21;
$time_zoom = 20;
$project_id = 0;
$kernel = undef;
$is_project = 0;
$is_compiled = 0;
my @sig = ();

# ---- DEBUG - Temporary values -----
if ($DEBUG) {
	my @sig = qw\BCD_A BCD_B BCD_C BCD_D END_RESET GATE F_INPUT F_PATTERN RESET START LED_A LED_B LED_C\;
	randomize;
	
	for (0 .. $#sig) {
		push @sigs, {
			name => $sig[$_],
			displayed => ($_ < 10) ? 1 : 0,
		};
		
		@xxx = ();
		my $time = 0;
		for my $j (0 .. 3) {
			$time += int(rand(5)) + 1;
			push @xxx, [$time, (0,1)[rand(2)]];
		}
	
		$sigs[$_]->{changes} = [@xxx];
	
		for my $k (0 .. 5) {
			if (defined $sigs[$_]->{changes}[$k]) {
			print "$sigs[$_]->{changes}[$k][0] = $sigs[$_]->{changes}[$k][1]  ($k) \n";
			}
		}	
	}
	
}
# ---- DEBUG END ----

# Create main window.
my $main = new MainWindow;

my $toplevel = $main->toplevel;
my $menubar = $toplevel->Menu(-type => 'menubar');
$toplevel->configure(-menu => $menubar);

my $modifier = 'Meta';	# Unix
my $modifier_key = 'Meta'; # Keyboard key
if ($^O eq 'MSWin32') {
	$modifier = 'Control';
	$modifier_key = 'Ctrl';
} elsif ($^O eq 'MacOS') {  # one of these days
	$modifier = 'Command';
	$modifier_key = 'Command';
}

# --- File menu ---
my $filemenu = $menubar->cascade(-label => '~File', -tearoff => 0);

$filemenu->command(-command => \&new_project,
    -label => "New...",
	-accelerator => "$modifier_key+N",
    -underline => 0);
$toplevel->bind("<$modifier-n>" => \&new_project);	

$filemenu->command(-command => \&open_project,
    -label => "Open...",
	-accelerator => "$modifier_key+O", 	
    -underline => 0);
$toplevel->bind("<$modifier-o>" => \&open_project);	
	
$filemenu->command(-command => \&save_project,
    -label => "Save",
	-accelerator => "$modifier_key+S", 		
    -underline => 0); 
$toplevel->bind("<$modifier-s>" => \&save_project);		
	
$filemenu->command(-command => \&save_as_project,
    -label => "Save As...",
    -underline => 5); 
	
$filemenu->command(-command => \&close_project,
    -label => "Close",
	-accelerator => "$modifier_key+W", 		
	-state => "disabled",	
    -underline => 0); 	
$toplevel->bind("<$modifier-w>" => \&close_project);			

$filemenu->separator;

$filemenu->command(-command => \&settings,
    -label => "Settings",
    -underline => 2); 	

$filemenu->command(-label => "Exit",
    -command => \&exit_choice,
    -underline => 1);  

# --- Compilation menu ---
my $compmenu = $menubar->cascade(-label => '~Compilation', -tearoff => 0);

$compmenu->command(-command => \&compile,
    -label => "Start compilation",
    -underline => 0);
	

# --- Simulation menu ---
my $simmenu = $menubar->cascade(-label => '~Simulation', -tearoff => 0);

$simmenu->command(-command => \&sim_run,
    -label => "Run",
    -underline => 0);

$simmenu->command(-command => \&sim_pause,
    -label => "Pause",
    -underline => 0);

$simmenu->command(-command => \&sim_stop,
    -label => "Stop",
    -underline => 0);

$simmenu->separator;

$simmenu->command(-command => \&sim_options,
    -label => "Simulation Options...",
    -underline => 11);
	
$simmenu->command(-command => \&sim_force,
    -label => "Force User Values...",
    -underline => 0);	
	
# --- Display menu ---
my $dispmenu = $menubar->cascade(-label => '~Display', -tearoff => 0);

$dispmenu->command(-command => \&disp_select,
    -label => "Select Signals",
    -underline => 0);	
	
# --- Help menu ---
my $helpmenu = $menubar->cascade(-label => '~Help', -tearoff => 0);

$helpmenu->command(-command => \&about_choice,
    -label => "About EasySim",
    -underline => 0); # A in About

# --------------


# ---- Shortcut Buttons ----
$buttons = $main->Frame(-relief=>"flat",
    -borderwidth=>2);

# -borderwidth=>1, -width=>20, -justify=>center, -relief=>flat	
my (@button_style) = qw/-borderwidth 1 -width 18 -height 18 -justify center -relief flat/;
my $img_new = $main->Photo(-file =>'data/new.gif');
$button_new = $buttons->Button(
	-image=>$img_new, 
	-command=>\&new_project, 
	@button_style);
$button_new->pack(-side=>"left");

my $img_open = $main->Photo(-file =>'data/open.gif');
$button_open = $buttons->Button(
	-image=>$img_open,
	-command=>\&open_project, 
	@button_style);
$button_open->pack(-side=>"left");

my $img_save = $main->Photo(-file =>'data/save.gif');
$button_save = $buttons->Button(
	-image=>$img_save,
	-command=>\&save_project, 
	@button_style);
$button_save->pack(-side=>"left");

my $img_div = $main->Photo(-file =>'data/div.gif');
$div = $buttons->Label(-image=>$img_div, -width=>4, -justify=>center);
$div->pack(-side=>"left");

my $img_comp = $main->Photo(-file =>'data/compile.gif');
$button_comp = $buttons->Button(
	-image=>$img_comp,
	-command=>\&compile, 
	@button_style);
$button_comp->pack(-side=>"left");

$div2 = $buttons->Label(-image=>$img_div, -width=>4, -justify=>center);
$div2->pack(-side=>"left");

my $img_run = $main->Photo(-file =>'data/run.gif');
$button_run = $buttons->Button(
	-image=>$img_run,
	-command=>\&sim_run, 
	@button_style);
$button_run->pack(-side=>"left");

my $img_pause = $main->Photo(-file =>'data/pause.gif');
$button_pause = $buttons->Button(
	-image=>$img_pause,
	-command=>\&sim_pause, 
	@button_style);
$button_pause->pack(-side=>"left");

my $img_stop = $main->Photo(-file =>'data/stop.gif');
$button_stop = $buttons->Button(
	-image=>$img_stop,
	-command=>\&sim_stop, 
	@button_style);
$button_stop->pack(-side=>"left");

my $img_div = $main->Photo(-file =>'data/div.gif');
$div = $buttons->Label(-image=>$img_div, -width=>4, -justify=>center);
$div->pack(-side=>"left");

# ---- Time scale ----
my $scale_entry = $buttons->BrowseEntry(-label => "Time scale: ", 
	-variable => \$scale,
	-width => 6,
	-borderwidth => 1,
	);
$scale_entry->pack(-side=>"left");
for ($i = 1; $i <= 3; $i+=1)
{
	$scale_entry->insert("end", 1 * 10 ** $i);
	$scale_entry->insert("end", 2 * 10 ** $i);
	$scale_entry->insert("end", 5 * 10 ** $i);
}

$buttons->pack(-side=>"top", -fill=>"x");
# -----------------


# ---- Content Canvas ----

#my $can = $main->Scrolled(qw/Canvas -width 600 -height 250 -relief groove
#			   -borderwidth 2 -scrollbars e -scrollregion/ => [qw/0c 0c 500 300/]);
#$can->pack(qw/-expand yes -fill both/);	

#my $img_content = $main->Photo(-file =>'data/content.gif');
#$can->createImage(0, 0, -image=>$img_content, -anchor=>"nw");

$frm_c = $main->Frame
	 (qw/
	   -relief groove
	   -borderwidth 1
	  /);
$frm_c->pack(qw/-expand yes -fill both/);	

$c = $frm_c->Scrolled
      (qw/HList
	   -relief groove
	   -borderwidth 0
	   -scrollbars se 
       -header 1
       -columns 3
	   -selectborderwidth 1
	   -selectbackground White
       -width 50
       -height 20/
      );
#  -background Grey	  
	  
$c->pack(qw/-expand yes -fill both/);	

$value_style = $c->ItemStyle('text');
$value_style->configure(
	-background => White,
	-pady => 5, 
	-padx => 15,

);
$signal_style = $value_style;

$window_style = $c->ItemStyle('window');
$window_style->configure(
	-pady => 0,
	-padx => 0,
);

@header_texts = ("Signal name", "Value", "");
for (0 .. $#header_texts) {
	$c->header('create', $_, -text => $header_texts[$_], -relief => ridge, -borderwidth=>1);
}

display_content();	# show me the way to the next whiskey bar (and don't ask why)
	
# ----------------------

# ----- Status area -----------
my $status = $main->Scrolled(qw/Text -relief sunken -borderwidth 2 -height 5 
 -setgrid true -scrollbars er/);

$status->pack(qw/-expand yes -fill both/);
# -----------------------------

print_status("EasySim GUI module started...");

# Let Perl/Tk handle window events.
MainLoop;


# --- Exit ---
sub exit_choice {
	if (defined $kernel) {
		$kernel->shutdown();
	}
    print "Exiting EasySim.\n";
    exit;
}

# --- New ---
sub new_project {
	if (defined($new_project_reference))
		{
		$new_project_reference->raise;
		$new_project_reference->focus;
		}
	else
		{
		my $pop = $main->Toplevel();
		$pop->title("New Project");
	
		my $fr_src = $pop->LabFrame(-label => "Source VHDL file", -labelside => "acrosstop");
		my $ent = $fr_src->Entry(-width => 25);
		my $but = $fr_src->Button(-text => "Browse...", 
			     -command => sub { new_project_dialog($pop, $ent);},
				 @BUTTON_STYLE);
		$ent->pack(-side => 'left',-expand => 'yes', -fill => 'x', @ENTRY_PACK_STYLE);
		$but->pack(-side => 'left', @BUTTON_PACK_STYLE);
		$fr_src->pack (@FRAME_PACK_STYLE);
		
		my $button_cancel = $pop->Button(text=>'Cancel', @BUTTON_STYLE,
			command => sub {
				$pop->destroy();
				$new_project_reference = undef;
			} )
			->pack(@BUTTON_PACK_STYLE, -side=>'right');
	
		my $button_ok = $pop->Button(text=>'OK', @BUTTON_STYLE,
			command => sub {
				new_project_create();
				$pop->destroy();
				$new_project_reference = undef;
				print_status("Creating new project (ID: $project_id).");
			} )
			->pack(@BUTTON_PACK_STYLE, -side=>'right');
					
		$pop->resizable('no','no');
		$new_project_reference = $pop;
	}
	
	sub new_project_create {
		my $msg=qq~<?xml version="1.0" encoding="iso8859-2"?>
			<easysim>
				<project id="0">
					<commands>
						<command name="newProject"/>
					</commands>
				</project>
			</easysim>
		~;
		my $parser=Easysim::XML->new();
		my $dom=$parser->parse_string("$msg");
		$dom_back = send_and_receive_message($dom);
		
		my $root = $dom_back->getDocumentElement();
		my $node_project = ($root->findnodes('/easysim/project'))[0];
		$project_id = $node_project->getAttribute("id");
		
		@sigs = undef;
		@sigs = ();
		display_content();
		
		$is_project = 1;
		$is_compiled = 0;
	}
	
	$is_project = 1;	# DEBUG -- window is hiding after double-click in open dialog
	$is_compiled = 0;
	
}

sub new_project_dialog {
    my $w = shift;		# widget
    my $ent = shift;	# entry item
	my $types;
    my $file;
    #   Type names		Extension(s)	Mac File Type(s)
    @types =
      (["VHDL Source files",           '.vhdl'],
       ["All files",		'*']
      );

	$file = $toplevel->getOpenFile(-filetypes => \@types);

    if (defined $file and $file ne '') {	
		$ent->delete(0, 'end');
		$ent->insert(0, $file);
		$ent->xview('end');
	}

	$w->raise;
	$w->focus;
}


# --- Open ---
sub open_project {
   	my $types;
    my $file;
    #   Type names		Extension(s)	Mac File Type(s)
    @types =
      (["EasySim Project files",           '.esp'],
       ["All files",		'*']
      );

	$file = $toplevel->getOpenFile(-filetypes => \@types);

    if (defined $file and $file ne '') {	
		print_status("Opening project $file.");
		$is_project = 1;
	}
}

# --- Save ---
sub save_project {

	print_status("Saving project.");
}

# --- Save as ---
sub save_as_project {
   	my $types;
    my $file;
    #   Type names		Extension(s)	Mac File Type(s)
    @types =
      (["EasySim Project files",           '.esp'],
       ["All files",		'*']
      );

	$file = $toplevel->getSaveFile(-filetypes => \@types,
				-initialfile => 'Untitled',
				-defaultextension => '.esp');

    if (defined $file and $file ne '') {	
		print_status("Saving project as $file.");
	}
}

# --- Close ---
sub close_project {

	print_status("Closing project.");
}

# --- Compile ---
sub compile{

	if (not $is_project) {
		print_status("Unable to compile: No project defined. Create or open an existing one.");
		die;
	}

	print_status("Starting compilation...");
	
	my $msg=qq~<?xml version="1.0" encoding="iso8859-2"?>
		<easysim>
			<project id="$project_id">
				<commands>
					<command name="compile">
						<vhdlSource>
							$vhdl_source
						</vhdlSource>
					</command>
				</commands>
			</project>
		</easysim>
	~;
	my $parser=Easysim::XML->new();
	my $dom=$parser->parse_string("$msg");
	$dom_back = send_and_receive_message($dom);
	print "\nMESSAGE:\n",$dom_back->toString,"\n";	
	
	@sigs = undef;
	@sigs = ();
	$root = $dom_back->getDocumentElement();
	for my $node_signal ($root->findnodes('/easysim/project/messages/compiler/msg/entity/architecture/process/signal')) {
		my $signal_name = $node_signal->getAttribute("name");
#		print($signal_name);
		push @sigs, {
			name => $signal_name,
			displayed => 1
		};
	}

	print_status("Compilation successful.");
	$is_compiled = 1;
	display_content();
}

# --- Run ---
sub sim_run{

	if (not $is_compiled) {
		print_status("Unable to simulate: Must compile first.");
		die;
	}

	print_status("Running simulation...");
	
	my $msg=qq~<?xml version="1.0" encoding="iso8859-2"?>
		<easysim>
			<project id="$project_id">
				<commands>
					<command name="simulate"/>
				</commands>
			</project>
		</easysim>
	~;
	my $parser=Easysim::XML->new();
	my $dom=$parser->parse_string("$msg");
	$dom_back = send_and_receive_message($dom);
	print "\nMESSAGE:\n",$dom_back->toString,"\n";	
	
	my $root = $dom_back->getDocumentElement();
	for my $node_signal ($root->findnodes('/easysim/project/messages/simulator/msg/entity/architecture/process/signal')) {
		$signal_name = $node_signal->getAttribute("name");
		
#		print "signal: $signal_name \n";
		
		$signal_pos = undef;
		for (0 .. $#sigs) {
			if ($sigs[$_]{name} eq $signal_name) {
				$signal_pos = $_;
				break;
			}
		}
		
		if (defined $signal_pos) {
			@changes = ();
		
			for my $node_change ($node_signal->findnodes('change')) {
				$change_time = $node_change->getAttribute("time");
				$change_value = $node_change->getAttribute("newValue");
				
				push @changes, [$change_time, $change_value];
			}
			$sigs[$signal_pos]->{changes} = [@changes];
		}		
	}

	display_content();
}

# --- Pause ---
sub sim_pause{

	print_status("Pausing simulation...");
}

# --- Compile ---
sub sim_stop{

	print_status("Stopping simulation...");
}

# --- Select Signals ---
sub disp_select{

	if (defined($disp_select_reference))
		{
		$disp_select_reference->raise;
		$disp_select_reference->focus;
		}
	else
		{

		# copy original values
		my @sigs_ds;
		@sigs_ds = copy_arrays(\@sigs, \@sigs_ds);
		
		my $pop = $main->Toplevel();
		$pop->title("Signals select");
		
		my $fr_sig = $pop->LabFrame(-label => "Displayed Signals", -labelside => "acrosstop");

		$select_c = $fr_sig->Scrolled
	      (qw/HList
		   -relief groove
		   -borderwidth 0
		   -indicator 1
		   -scrollbars e
	       -header 0
	       -columns 2
		   -selectborderwidth 1 
	       -width 30
	       -height 15/,
		   -browsecmd => sub {}
       );
	  
		$select_c->pack(qw/-expand yes -fill both/);	
		
		$select_signal_style = $select_c->ItemStyle('text');
		$select_signal_style->configure(
			-pady => 3, 
			-padx => 5,
		);

		my $select_window_style = $select_c->ItemStyle('window');
		$select_window_style->configure(
			-pady => 0,
			-padx => 0,
		
		);
		
		my $select_c_line = 0;

		for my $i (0 .. $#sigs_ds) {
			
			$select_c_line++;
			$select_c_item = $select_c->addchild("");
			
			$select_c->itemCreate
			      ($select_c_item, 0,
			       -style => $select_signal_style,
				   -text => "$sigs_ds[$i]{name}",
			      );
		
			push @select_box, $select_c->Checkbutton(
				-variable => \$sigs_ds[$i]{displayed}
			);					
		
			$select_c->itemCreate
			      ($select_c_item, 1,
			        -style => $select_window_style,
					-itemtype => 'window',   
					-widget => $select_box[$i]  		
			      );

		}

		$select_c->columnWidth(0, "4c");
		
		my $button_select_all = $fr_sig->Button(text=>'Select All', @BUTTON_STYLE,
			command => sub {
				for (0 .. $#sigs_ds) {$sigs_ds[$_]{displayed} = 1;}
			} )
			->pack(@BUTTON_PACK_STYLE, -side=>'left');
			
    	my $button_unselect_all = $fr_sig->Button(text=>'Unselect All', @BUTTON_STYLE,
			command => sub {
				for (0 .. $#sigs_ds) {$sigs_ds[$_]{displayed} = 0;}
			} )
			->pack(@BUTTON_PACK_STYLE, -side=>'right');			
		
		$fr_sig->pack (@FRAME_PACK_STYLE);
		
		my $button_cancel = $pop->Button(text=>'Cancel', @BUTTON_STYLE,
			command => sub {
				$pop->destroy();
				$disp_select_reference = undef;
			} )
			->pack(@BUTTON_PACK_STYLE, -side=>'right');
	
		my $button_ok = $pop->Button(text=>'OK', @BUTTON_STYLE,
			command => sub {
				$pop->destroy();
				$disp_select_reference = undef;
				@sigs = copy_arrays(\@sigs_ds, \@sigs); # store modified signal values
				display_content(); # redraw signals
			} )
			->pack(@BUTTON_PACK_STYLE, -side=>'right');		
				
		$pop->resizable('no','no');
		$disp_select_reference = $pop;
		}


}

# --- Simulation Options ---
sub sim_options{
	if (defined($sim_options_reference))
		{
		$sim_options_reference->raise;
		$sim_options_reference->focus;
		}
	else
		{
		
		my $pop = $main->Toplevel();
		$pop->title("Simulation Options");
		
		my $fr_time = $pop->LabFrame(-label => "Simulation Time", -labelside => "acrosstop");
		
		my $time_ent = $fr_time->Entry(-width => 12);
		$time_ent->pack(-side => 'left',-expand => 'yes', -fill => 'x', @ENTRY_PACK_STYLE);
		$time_ent->insert('end', $sim_time);

		$fr_time->pack (@FRAME_PACK_STYLE);
		
		my $fr_sen = $pop->LabFrame(-label => "Signal Sensitivity", -labelside => "acrosstop");
		$fr_sen->pack (@FRAME_PACK_STYLE);
		
		my $button_cancel = $pop->Button(text=>'Cancel', @BUTTON_STYLE,
			command => sub {
				$pop->destroy();
				$sim_options_reference = undef;
			} )
			->pack(@BUTTON_PACK_STYLE, -side=>'right');
	
		my $button_ok = $pop->Button(text=>'OK', @BUTTON_STYLE,
			command => sub {
				 # save values
				$sim_time = $time_ent->get();
			
				$pop->destroy();
				$sim_options_reference = undef;		
			} )
			->pack(@BUTTON_PACK_STYLE, -side=>'right');		
	
		$pop->resizable('no','no');
		$sim_options_reference = $pop;
		}


}

# --- Settings ---
sub settings{
	if (defined($settings_reference))
		{
		$settings_reference->raise;
		$settings_reference->focus;
		}
	else
		{
		
		my $pop = $main->Toplevel();
		$pop->title("EasySim Settings");
		
		my $fr_kernel = $pop->LabFrame(-label => "Kernel Address", -labelside => "acrosstop");
		
		my $kernel_ent = $fr_kernel->Entry(-width => 12);
		$kernel_ent->pack(-side => 'left',-expand => 'yes', -fill => 'x', @ENTRY_PACK_STYLE);
		$kernel_ent->insert('end', $KERNEL_SERVER);

		$fr_kernel->pack (@FRAME_PACK_STYLE);
		
	
		my $button_cancel = $pop->Button(text=>'Cancel', @BUTTON_STYLE,
			command => sub {
				$pop->destroy();
				$settings_reference = undef;
			} )
			->pack(@BUTTON_PACK_STYLE, -side=>'right');
	
		my $button_ok = $pop->Button(text=>'OK', @BUTTON_STYLE,
			command => sub {
				 # save values
				$KERNEL_SERVER = $kernel_ent->get();
				$kernel = undef; # set back to undefined
				
				print_status("Kernel address is now $KERNEL_SERVER.");
			
				$pop->destroy();
				$settings_reference = undef;		
			} )
			->pack(@BUTTON_PACK_STYLE, -side=>'right');		
	
		$pop->resizable('no','no');
		$settings_reference = $pop;
		}


}

# --- About box ---
sub about_choice {
    $main->messageBox(qw/-icon info -type OK -message/ => "EasySim" . chr(10). "(c) 2001 Team 08" . chr(10) . chr(10). "http://www2.dcs.elf.stuba.sk/~team08/", -title=>"About EasySim" );
}

# --- Common functions ---

# ---- Redraws main content area ----
sub display_content {
	$c->delete(all);

	my $c_line = 0;
	$canvas_width = $time_zoom*$sim_time;

	for my $i (0 .. $#sigs) {
		if ($sigs[$i]{displayed}) {	
			$c_line++;
			$c_item = $c->addchild("");
			
			$c->itemCreate
			      ($c_item, 0,
			       -style => $signal_style,
				   -text => "$sigs[$i]{name}",
			      );
		
			$c->itemCreate
			      ($c_item, 1,
			       -style => $value_style,
				   -text => "$sigs[$i]{value}",
				  
			      );
				  
			$cans[$i] = $c->Canvas(
				qw/
				-borderwidth 0
				-background White
				-highlightthickness 0			
			/, 	-height => $SIGNAL_HEIGHT,
				-width => $canvas_width
			);
			
			if (defined $sigs[$i]{changes}) {
				my $x1 = 0;
				my $y = 0;
				$y_pos = $SIGNAL_HEIGHT - ($y * $SIGNAL_REAL_HEIGHT + $SIGNAL_PADDING);
				for my $k (0 .. 5) {
					if (defined $sigs[$i]->{changes}[$k]) {
#						print "$sigs[$i]->{changes}[$k][0] = $sigs[$i]->{changes}[$k][1]  ($k) \n";
						$x2 = $sigs[$i]{changes}[$k][0] * $time_zoom;
						$old_y_pos = $y_pos;
						$y_pos = $SIGNAL_HEIGHT - ($y * $SIGNAL_REAL_HEIGHT + $SIGNAL_PADDING);
						$cans[$i]->createLine($x1, $y_pos, $x2, $y_pos, -fill=>$SIGNAL_COLOR);
						$cans[$i]->createLine($x1, $y_pos, $x1, $old_y_pos, -fill=>$SIGNAL_COLOR);
						
						$y = $sigs[$i]{changes}[$k][1];
						
						$x1 = $x2;
					}
				}
				
				$old_y_pos = $y_pos;
				$y_pos = $SIGNAL_HEIGHT - ($y * $SIGNAL_REAL_HEIGHT + $SIGNAL_PADDING);
				$x2 = $sim_time * $time_zoom;
				$cans[$i]->createLine($x1, $y_pos, $x2, $y_pos, -fill=>$SIGNAL_COLOR);
				$cans[$i]->createLine($x1, $y_pos, $x1, $old_y_pos, -fill=>$SIGNAL_COLOR);
			}
				  
			$c->itemCreate
			      ($c_item, 2,
			       -style => $window_style,
				   -itemtype => 'window',   
				   -widget => $cans[$i]  
			      );		  
		
		}		  
	}
	
	$c->columnWidth(0, "3c");
	
}

# --- Prints text to status area ---
sub print_status {
	my $text = shift;

    # Fill in status area.
    $status->insert('end', $text . chr(10));
    $status->yview(moveto,1);
}

# --- Copy content of array a1 to array a2 ---
sub copy_arrays {
	my ($ref_a1, $ref_a2) = @_;
	@a1 = @$ref_a1;
	@a2 = @$ref_a2;
	
	splice(@a2, 0, $#a2);

	for my $i (0 .. $#a1) { 
		for my $k ( keys %{ $a1[$i] } ) {
			$a2[$i]{$k} = $a1[$i]{$k};
		}
	}	

	return(@a2);
}

sub send_and_receive_message {
	$dom = shift;
	
	if (not defined $kernel) {
		# --- Connection to kernel module ---
		$kernel=new Easysim::Communication("client","$KERNEL_SERVER:$config->{'port-gui'}");
	}

	$kernel->send_message($dom);
	my $dom_back = undef;
	do {
		$dom_back = $kernel->receive_message();
	} while(not defined $dom_back or $dom_back eq "");
	
	return($dom_back);
}

